/*
 * Copyright (C) 2005 - 2012 Jaspersoft Corporation. All rights reserved.
 * http://www.jaspersoft.com.
 * Unless you have purchased  a commercial license agreement from Jaspersoft,
 * the following license terms  apply:
 *
 * This program is free software: you can redistribute it and/or  modify
 * it under the terms of the GNU Affero General Public License  as
 * published by the Free Software Foundation, either version 3 of  the
 * License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU Affero  General Public License for more details.
 *
 * You should have received a copy of the GNU Affero General Public  License
 * along with this program. If not, see <http://www.gnu.org/licenses/>.
 */

package com.jaspersoft.jasperserver.jaxrs.common;

import com.jaspersoft.jasperserver.remote.exception.IllegalParameterValueException;
import com.jaspersoft.jasperserver.remote.exception.builders.LocalizedErrorDescriptorBuilder;
import com.jaspersoft.jasperserver.remote.exception.xml.ErrorDescriptor;
import org.hibernate.JDBCException;
import org.hibernate.exception.DataException;
import org.springframework.dao.DataIntegrityViolationException;
import org.springframework.stereotype.Component;

import javax.annotation.Resource;
import javax.ws.rs.core.Response;
import javax.ws.rs.ext.ExceptionMapper;
import javax.ws.rs.ext.Provider;

/**
 * <p></p>
 *
 * @author Zakhar.Tomchenco
 * @version $Id$
 */
@Provider
@Component
public class DataIntegrityViolationExceptionMapper implements ExceptionMapper<DataIntegrityViolationException> {

    @Resource(name= "localizedErrorDescriptorBuilder")
    private LocalizedErrorDescriptorBuilder builder;


    @Override
    public Response toResponse(DataIntegrityViolationException exception) {
        ErrorDescriptor descriptor;
        if (exception.getCause() instanceof JDBCException) {
            JDBCException cause = (JDBCException) exception.getCause();
            cause.getSQLException().getNextException();
            descriptor = new ErrorDescriptor.Builder()
                    .setErrorCode(IllegalParameterValueException.ERROR_CODE)
                    .setParameters("", "", cause.getSQLException().getNextException() == null ?
                            cause.getSQLException().getLocalizedMessage() :
                            cause.getSQLException().getNextException().getLocalizedMessage())
                    .getErrorDescriptor();
        } else {
            descriptor = new ErrorDescriptor.Builder()
                    .setErrorCode(IllegalParameterValueException.ERROR_CODE)
                    .setParameters("", "",  exception.getMostSpecificCause().getLocalizedMessage())
                    .getErrorDescriptor();
        }

        return Response.status(Response.Status.BAD_REQUEST)
                .entity(builder.localizeDescriptor(descriptor))
                .build();
    }
}
